@@ -23,10 +23,10 @@
 #include <gtk/gtk.h>
 
 #include "giggle-branches-view.h"
-#include "giggle-git-refs.h"
-#include "giggle-ref.h"
-#include "giggle-job.h"
-#include "giggle-git.h"
+#include "libgiggle/giggle-git-refs.h"
+#include "libgiggle/giggle-ref.h"
+#include "libgiggle/giggle-job.h"
+#include "libgiggle/giggle-git.h"
 
 typedef struct GiggleBranchesViewPriv GiggleBranchesViewPriv;
 
@@ -77,7 +77,7 @@ branches_view_job_callback (GiggleGit *git,
 						 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_OK,
-						 _("An error ocurred when retrieving branches list:\n%s"),
+						 _("An error occurred when retrieving branches list:\n%s"),
 						 error->message);
 
 		gtk_dialog_run (GTK_DIALOG (dialog));
@@ -86,7 +86,7 @@ branches_view_job_callback (GiggleGit *git,
 		GtkListStore *store;
 		GtkTreeIter   iter;
 
-		store = giggle_short_list_get_liststore (GIGGLE_SHORT_LIST (view));
+		store = gtk_list_store_new (GIGGLE_SHORT_LIST_N_COLUMNS, G_TYPE_OBJECT);
 		branches = giggle_git_refs_get_branches (GIGGLE_GIT_REFS (job));
 
 		for(; branches; branches = g_list_next (branches)) {
@@ -95,6 +95,9 @@ branches_view_job_callback (GiggleGit *git,
 					    GIGGLE_SHORT_LIST_COL_OBJECT, branches->data,
 					    -1);
 		}
+
+		giggle_short_list_set_model (GIGGLE_SHORT_LIST (view), GTK_TREE_MODEL (store));
+		g_object_unref (store);
 	}
 
 	g_object_unref (priv->job);
@@ -108,8 +111,6 @@ branches_view_update (GiggleBranchesView *view)
 
 	priv = GET_PRIV (view);
 
-	gtk_list_store_clear (giggle_short_list_get_liststore (GIGGLE_SHORT_LIST (view)));
-
 	if (priv->job) {
 		giggle_git_cancel_job (priv->git, priv->job);
 		g_object_unref (priv->job);
@@ -142,9 +143,10 @@ giggle_branches_view_init (GiggleBranchesView *view)
 	g_signal_connect_swapped (priv->git, "notify::git-dir",
 				  G_CALLBACK (branches_view_update), view);
 
-	branches_view_update (view);
-
 	g_object_set (view, "label", _("Branches:"), NULL);
+
+	/* initialize for first time */
+	branches_view_update (view);
 }
 
 static void
