@@ -23,7 +23,7 @@
 
 #include <glib-object.h>
 
-#include "giggle-revision.h"
+#include "libgiggle/giggle-revision.h"
 
 G_BEGIN_DECLS
 
@@ -45,24 +45,24 @@ struct _GiggleFileListClass {
 	GtkTreeViewClass parent_class;
 
 	void (* path_selected) (GiggleFileList *list, const gchar *path);
+	void (* project_loaded) (GiggleFileList *list);
+	void (* status_changed) (GiggleFileList *list);
 };
 
-GType              giggle_file_list_get_type          (void);
-GtkWidget *        giggle_file_list_new               (void);
+GType              giggle_file_list_get_type            (void);
+GtkWidget *        giggle_file_list_new                 (void);
 
-gboolean           giggle_file_list_get_show_all      (GiggleFileList *list);
-void               giggle_file_list_set_show_all      (GiggleFileList *list,
-						       gboolean        show_all);
+gboolean           giggle_file_list_get_show_all        (GiggleFileList *list);
+void               giggle_file_list_set_show_all        (GiggleFileList *list,
+							 gboolean        show_all);
 
 void               giggle_file_list_highlight_revisions (GiggleFileList *list,
 							 GiggleRevision *from,
 							 GiggleRevision *to);
 
-GList *            giggle_file_list_get_selection     (GiggleFileList *list);
-
-gboolean           giggle_file_list_get_compact_mode  (GiggleFileList *list);
-void               giggle_file_list_set_compact_mode  (GiggleFileList *list,
-						       gboolean        compact_mode);
+GList *            giggle_file_list_get_selection       (GiggleFileList *list);
+void		   giggle_file_list_select              (GiggleFileList *list,
+							 const char     *path);
 
 G_END_DECLS
 
